/*
 * #(@)QuagentControl.java
 *
 * dave costello, costello@cs.rochester.edu, 28 Jan 2004
 * Time-stamp: <2005-02-28 11:48:44 costello>
 *
 * A very simple quagent client.
 * Connects to quake quagent socket (usually port 33333), 
 * Provides minimalist directional control buttons to move
 * quagent around quake world.
 * Test quagent protocol text commands by typing them in and hitting
 * "Send to Bot" button (or just hit enter).
 * A more sophisticated quagent client would employ much more than
 * just steering controls but would include some
 * understanding of the quagent messaging protocol and might make use of
 * something like the Jess inference engine to perform reasoning on 
 * quagent feedback from the quake world. 
 */

import java.net.*;
import java.io.*;
import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.ImageIcon;

import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.JScrollPane;
import javax.swing.JScrollBar;
import javax.swing.JPanel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

public class QuagentControl implements ActionListener, KeyListener {
    //
    // attributes

    /** host where quagent server is running */
    private String host         = "localhost";
    /** quagent server port */
    private int port            = 33333;
    private String resourcePath = "resources/images";
    private String title        = "Quake Agent Controller";
    private Socket sock       = null;
    private BufferedReader in = null;
    private PrintWriter out   = null;
    private String msg;

    private JFrame buttonFrame;
    private JFrame msgFrame;
    private JPanel msgPanel;
    private JPanel inputPanel;
    private JTextArea msgArea;
    private JTextArea inputPane;
    private JScrollBar scrollbar;

    /**
     * contructor
     */
    public QuagentControl(String[] args){
	super();
	handleCmdLine(args);
	initButtons();
	initMsgFrame();
	// listener for frame close event
        buttonFrame.addWindowListener(new WindowAdapter() {
             public void windowClosing(WindowEvent e) {
		 quit();
             }});
	//connect to quake and spawn bot
	connectQuagent();
    }
    /**
     * close sockets and quit.  This will kill your quagent bot in the game.
     */
    private void quit(){
	System.out.println("quitting...");
	buttonFrame.setVisible(false);
	msgFrame.setVisible(false);
	try{
	    //clean up
	    out.close();
	    in.close();
	    sock.close();
	}catch (Exception e){
	    System.out.println("Socket Close Exception:" + e.getMessage());
	}
	buttonFrame.dispose();
	msgFrame.dispose();
	System.out.println("socket closed.");
	System.exit(0);
    }
    /**
     * initializes message window
     */
    private void initMsgFrame(){
	// set location bottom right of screen
	GeometrySpec gs = new GeometrySpec("-0-0");
	msgFrame = new JFrame("Messages Received From Quagent Bot");
	msgPanel = new JPanel();
	inputPanel = new JPanel();
	msgArea = new JTextArea(10,80);//rows,cols
	inputPane = new JTextArea(1,70);//rows,cols
	// we will listen for return key
	inputPane.addKeyListener(this);


	JButton sendButton = new JButton("Send to Bot");
        sendButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    sendMsgToBot();
                }});

	JScrollPane scroller = new JScrollPane(msgArea);
	// we'll use the scroll bar to set max value and auto scroll when text is added
	scrollbar = scroller.getVerticalScrollBar();
	// Add scrollpane to msgPanel
        msgPanel.add(scroller, BorderLayout.CENTER); 
	// add input pane and send button
	inputPanel.add(inputPane);
	inputPanel.add(sendButton);
	// add panels to frame
	msgFrame.getContentPane().add(msgPanel, BorderLayout.NORTH);
	msgFrame.getContentPane().add(inputPanel, BorderLayout.SOUTH);
	msgFrame.pack();
	gs.setLocationOfFrame(msgFrame);
	msgFrame.setVisible(true);
    }
    /**
     * initializes control buttons
     */    
    private void initButtons(){
        JButton upButton, downButton, leftButton, rightButton,stopButton;
	ImageIcon upIcon, downIcon, leftIcon, rightIcon, stopIcon;

	upIcon = new ImageIcon(getClass().getResource(resourcePath+"/up.gif"));
	downIcon = new ImageIcon(getClass().getResource(resourcePath+"/down.gif"));
	leftIcon = new ImageIcon(getClass().getResource(resourcePath+"/left.gif"));
	rightIcon = new ImageIcon(getClass().getResource(resourcePath+"/right.gif"));
	stopIcon = new ImageIcon(getClass().getResource(resourcePath+"/stop.gif"));			  
	upButton    = new JButton(upIcon);
        downButton  = new JButton(downIcon);
	leftButton  = new JButton(leftIcon);
	rightButton = new JButton(rightIcon);
	stopButton  = new JButton(stopIcon);			  

	upButton.setBackground(Color.WHITE);
	downButton.setBackground(Color.WHITE);
	leftButton.setBackground(Color.WHITE);
	rightButton.setBackground(Color.WHITE);
	stopButton.setBackground(Color.WHITE);
	
        upButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    moveForward();
                }});
	
        downButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    moveBackward();
                }});

        leftButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    moveLeft();
                }});
        rightButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    moveRight();
                }});
        stopButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    stopMoving();
                }});

	buttonFrame = new JFrame(title);
        buttonFrame.getContentPane().add(upButton,BorderLayout.NORTH);
        buttonFrame.getContentPane().add(downButton,BorderLayout.SOUTH);	
	buttonFrame.getContentPane().add(stopButton,BorderLayout.CENTER);	
        buttonFrame.getContentPane().add(leftButton,BorderLayout.WEST);
        buttonFrame.getContentPane().add(rightButton,BorderLayout.EAST);
	buttonFrame.pack();
	buttonFrame.setVisible(true);

    }
    /**
     * Connect to quagent server to spawn quagent bot and begin listening
     * for and sending quagent messages.
     */    
    private void connectQuagent(){
	debug("connecting quagent...");
	try{
	    //init socket connection and streams
	    sock = new Socket(host,port);
	    in =new BufferedReader(new InputStreamReader(sock.getInputStream()));
	    out = new PrintWriter(sock.getOutputStream(),true);

	    // startup incoming msg reader thread
	    new ReadMsgsThread(in,this).start();
	    
	} catch (UnknownHostException e1) {
	    System.out.println("UnknownHostException:" + e1.getMessage());
	} catch (ConnectException e2){
	    System.out.println("ConnectException:" + e2.getMessage());
	} catch (SocketException e3){
	    System.out.println("SocketException:" + e3.getMessage());
	} catch (IOException e4){
	    System.out.println("IO Error:" + e4.getMessage());
	}catch (Exception e5){ 
	    System.out.println("General Exception:" + e5.getMessage());
	    System.out.println("closing socket...");
	    try{
		out.close();
		in.close();
		sock.close();
	    }catch (Exception e6){
		System.out.println("Close Exception:" + e6.getMessage());
	    }
	}
    }
    /**
     * prints a received message to the messages window 
     */
    protected void rcvMsg(String msg){
	msgArea.append("<recd>> "+msg);
	msgArea.append("\n");
	//auto scroll to bottom of window
	scrollbar.setValue(scrollbar.getMaximum());
    } 
		
    //
    // button actions defined 
    //

    /**
     * sends contents of inputPane down the socket to the quagent bot
     */
    private void sendMsgToBot(){
	String cmd = inputPane.getText();
	out.println(cmd);
	msgArea.append("<<sent> "+cmd);
	msgArea.append("\n");
	inputPane.setText("");
    }

    private void moveForward(){
	debug("moveForward()");
	out.println("do walkby 2000");
    }
    private void moveBackward(){
	debug("moveBackward()");
	out.println("do turnby 180");
	out.println("do walkby 2000");
    }
    private void moveLeft(){
	debug("moveLeft()");
	out.println("do turnby 90");
	out.println("do walkby 2000");
    }
    private void moveRight(){
	debug("moveRight()");
	out.println("do turnby -90");
	out.println("do walkby 2000");
    }
    private void stopMoving(){
	debug("stopMoving()");
	out.println("do stand");
    }
    public static void debug(String str){
	System.err.println(str);
    }
    public void actionPerformed(ActionEvent e){
        System.err.println("actionPerformed()");
    }
    public void keyTyped(KeyEvent e) {}
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {
	int keycode = e.getKeyCode();
	// Did someone hit the enter key?
	if (keycode == KeyEvent.VK_ENTER) {	
	    sendMsgToBot();
	}
    }

    /**
     * parse commandline arguments
     */
    public void handleCmdLine(String[] args) {
        int i=0;
        String arg;

        while ( i < args.length ) {
            if (args[i].startsWith("-")) {
                arg = args[i++];

                if(arg.equals("-port")) {
                    this.port = Integer.parseInt(args[i]);
                    //debug("cmd line setting port:"+ port);
                    i++;
                }
                if(arg.equals("-host")) {
                    this.host = args[i];
                    //debug("cmd line -host "+ host);
                    i++;
                }
	    }
	}
    }
    /**
     * Application main
     */
    public static void main(String args[]){
      new QuagentControl(args);
    }
}

/**
 * Listens to socket connected to quagent and reads incoming messages
 */
class ReadMsgsThread extends Thread {
    BufferedReader in;
    QuagentControl qc;
    String msg;
    /**
     * constructor
     */
    public ReadMsgsThread(BufferedReader in, QuagentControl qc){
	this.in = in;
	this.qc = qc;
    }
    /**
     * main execution loop for thread
     */
    public void run(){
	try{
	    // read until end of stream or error
	    while(true){
		// read if we don't have to block
		if (in.ready()){
		    msg = in.readLine();
		    qc.rcvMsg(msg);
		}
	    }
	}catch (IOException ioe){
	    System.out.println("Socket IO: "+ioe.getMessage());
	}
    }
}
