import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.charset.*;

/**
 * Gestione del socket di comunicazione tra l'agente e l'infrastruttura Quagent.
 */
public class QuagentSocket {
	
	SocketChannel sChannel;

    /**
     * Costruttore della classe.
     * @param s Oggetto SocketChannel contenente i parametri di connessione
     */
	public QuagentSocket (SocketChannel s) {
	    sChannel = s;
	    // Si assicura che il canale passato come parametro sia realmente connesso
	    // prima di procedere
	    try {
	    	while (!sChannel.finishConnect())
	    		;
	    } catch (IOException e) {
	    	System.out.println("IO Error:"+e.getMessage());
	    }
	}

    /**
     * Invia la stringa passata come argomento sul canale di comunicazione.
     * @param msg La stringa da inviare sul canale.
     * @return Il numero di byte inviati sul canale, 0 in caso di fallimento.
     */
    public int sendString(String msg) {
    	int numBytesWritten = 0;
    	try {
    		ByteBuffer buf = ByteBuffer.wrap( msg.getBytes() ); 
    		numBytesWritten = sChannel.write(buf);
    	} catch (IOException e) {
    		System.out.println("IO Error:"+e.getMessage());
    	}
    	return numBytesWritten;
    }

    /**
     * Riceve una stringa dal canale di comunicazione.
     * @return La stringa ricevuta dal canale o una stringa vuota ("") in caso di fallimento.
     */
    public String receiveString() {
    	ByteBuffer buf = ByteBuffer.allocate(1024);
    	int numBytesRead = 0;
    	String msg=null;
    	try {
    		buf.clear();
    		numBytesRead = sChannel.read(buf);
    		if(numBytesRead < 0) 
    			return null;
    		if(numBytesRead == 0) 
    			return "";
    		buf.flip();
    		Charset charset=Charset.forName("US-ASCII");
    		CharsetDecoder decoder = charset.newDecoder();
    		CharBuffer charBuffer = decoder.decode(buf);
    		msg = new String(charBuffer.toString());
    	} catch (IOException e){
    		System.out.println("IO Error:" + e.getMessage());
    	}
    	return msg;
    }

    /**
     * Legge il numero di byte indicati dal canale di comunicazione.
     * @param msg Array di byte in cui vengono salvati i byte letti.
     * @param len Numero di byte da leggere.
     * @return Il numero di byte effettivamente letti, 0 in caso di fallimento.
     */
    public int read(byte[] msg, int len) {
    	ByteBuffer buf = ByteBuffer.allocate(len);
    	int numTotalRead = 0;
    	try {
    		buf.clear();
    		int numBytesRead = 0;
    		while(numTotalRead < len) {
    			numBytesRead=sChannel.read(buf);
    			if(numBytesRead <= 0) 
    				return numBytesRead;
    			numTotalRead += numBytesRead;
    		}
    		buf.flip();
    		buf.get(msg);
    	} catch (IOException e){
    		System.out.println("IO Error:" + e.getMessage());
    	}
    	return numTotalRead;
    }
    
    /**
     * Legge una riga (tutti i caratteri disponibili fino al successivo carattere di fine linea) dal
     * canale di comunicazione,
     * @return La stringa letta dal canale, null in caso di fallimento.
     */
    public String getLine() {
    	String msg = null;
    	StringBuffer line = new StringBuffer();
    	ByteBuffer buf = ByteBuffer.allocate(1);
    	byte c=0; 
    	try {
    		int numBytesRead;
    		do {
    			do {
    				buf.clear();
    				numBytesRead = sChannel.read(buf);
    			} while(numBytesRead == 0);
    			if(numBytesRead < 0) 
    				break;
    			buf.flip();
    			c = buf.get(0);
    			if(c != 10 && c != 13) {
    				line.append((char)c);
    			}
    			buf.clear();
    		} while (c!=10 && c != 13);
    		if (numBytesRead < 0) 
    			return null;
    		msg = line.toString();
    	} catch (IOException e){
    		System.out.println("IO Error:" + e.getMessage());
    	}
    	return msg;
    }
    
    /**
     * Scrive il numero di byte indicati sul canale di comunicazione.
     * @param msg Array contenente i byte da scrivere sul canale.
     * @param len Numero di byte da scrivere sul canale (dimensione dell'array msg).
     * @return Il numero di byte effettivamente scritti, 0 in caso di fallimento.
     */
    public int write(byte[] msg, int len) {
    	ByteBuffer buf = ByteBuffer.allocate(len);
    	buf.clear();
    	buf.put(msg);
    	int numTotalWritten= 0;
    	try {
    		int numBytesWritten= 0;
    		while(numTotalWritten< len) {
    			numBytesWritten=sChannel.write(buf);
    			if(numBytesWritten <= 0)
    				return numBytesWritten;
    			numTotalWritten += numBytesWritten;
    		}
    	} catch (IOException e){
    		System.out.println("IO Error:" + e.getMessage());
    	}
    	return numTotalWritten;
    }
}


